@@ -1,22 +1,21 @@
 from . import data_loaders
-from .functions_sdk.function_errors import InvocationError
+from .functions_sdk.exceptions import RequestException
 from .functions_sdk.functions import (
-    GraphInvocationContext,
+    GraphRequestContext,
     RouteTo,
     TensorlakeCompute,
     tensorlake_function,
 )
 from .functions_sdk.graph import Graph
+from .functions_sdk.http_client import TensorlakeClient
 from .functions_sdk.image import Image
+from .functions_sdk.remote_graph import RemoteGraph
 from .functions_sdk.retries import Retries
-from .http_client import TensorlakeClient
-from .remote_graph import RemoteGraph
-from .settings import DEFAULT_SERVICE_URL
 
 __all__ = [
     "data_loaders",
     "Graph",
-    "GraphInvocationContext",
+    "GraphRequestContext",
     "Image",
     "RemoteGraph",
     "Pipeline",
@@ -25,7 +24,6 @@ __all__ = [
     "RouteTo",
     "tensorlake_function",
     "TensorlakeCompute",
-    "DEFAULT_SERVICE_URL",
     "TensorlakeClient",
-    "InvocationError",
+    "RequestException",
 ]
