@@ -27,7 +27,6 @@ from httpx_sse import aconnect_sse
 from pydantic import BaseModel
 
 from tensorlake import Image
-import signal
 
 
 @dataclass
@@ -93,6 +92,7 @@ class BuildLogEvent(BaseModel):
     stream: str
     message: str
     sequence_number: int
+    build_status: str
 
 
 class ImageBuilderV2Client:
@@ -194,7 +194,15 @@ class ImageBuilderV2Client:
             timeout=60,
         )
 
-        res.raise_for_status()
+        if not res.is_success:
+            error_message = res.text
+            click.secho(
+                f"Error building image {image.image_name}: {error_message}", fg="red"
+            )
+            raise RuntimeError(
+                f"Error building image {image.image_name}: {error_message}"
+            )
+
         build = BuildInfo.model_validate(res.json())
 
         click.secho(f"Starting build for image {image.image_name} ...", fg="green")
@@ -233,23 +241,28 @@ class ImageBuilderV2Client:
             ) as event_source:
                 async for sse in event_source.aiter_sse():
                     log_entry = BuildLogEvent.model_validate(sse.json())
-                    if log_entry.stream == "stdout":
-                        click.secho(
-                            log_entry.message,
-                            nl=False,
-                            err=False,
-                            fg="black",
-                            dim=True,
-                        )
-                    elif log_entry.stream == "stderr":
-                        click.secho(log_entry.message, fg="red", err=True)
-                    elif log_entry.stream == "info":
-                        click.secho(
-                            f"{log_entry.timestamp}: {log_entry.message}", fg="blue"
-                        )
+                    self._print_build_log_event(event=log_entry)
 
         return await self.build_info(build.id)
 
+    def _print_build_log_event(self, event: BuildLogEvent):
+        if event.build_status == "pending":
+            click.secho("Build waiting in queue...", fg="yellow")
+        else:
+            match event.stream:
+                case "stdout":
+                    click.secho(
+                        event.message,
+                        nl=False,
+                        err=False,
+                        fg="black",
+                        dim=True,
+                    )
+                case "stderr":
+                    click.secho(event.message, fg="red", err=True)
+                case "info":
+                    click.secho(f"{event.timestamp}: {event.message}", fg="blue")
+
     async def build_info(self, build_id: str) -> BuildInfo:
         """
         Get information about a build.
@@ -264,7 +277,11 @@ class ImageBuilderV2Client:
             headers=self._headers,
             timeout=60,
         )
-        res.raise_for_status()
+        if not res.is_success:
+            error_message = res.text
+            click.secho(f"Error building image: {error_message}", fg="red")
+            raise RuntimeError(f"Error building image: {error_message}")
+
         build_info = BuildInfo.model_validate(res.json())
 
         if build_info.status == "failed":
@@ -291,13 +308,12 @@ class ImageBuilderV2Client:
 
             if response.status_code == 202:
                 click.secho(
-                    f"Build for image {image.image_name} cancelled successfully", fg="green"
+                    f"Build for image {image.image_name} cancelled successfully",
+                    fg="green",
                 )
             else:
                 click.secho(f"Failed to cancel build {build.id}", fg="red")
-            click.secho(
-                f"Cancelled build for image {image.image_name}", fg="yellow"
-            )
+            click.secho(f"Cancelled build for image {image.image_name}", fg="yellow")
         except Exception as e:
             click.secho(
                 f"Failed to cancel build for image {image.image_name}: {e}",
