@@ -4,9 +4,12 @@ import asyncio
 import inspect
 import json
 import time
-from typing import Any, Dict, List, Mapping, Optional, Union
+from typing import Any, Callable, Dict, List, Mapping, Optional, Union
 
-from pydantic import BaseModel
+from httpx_sse import ServerSentEvent, aconnect_sse, connect_sse
+from pydantic import BaseModel, ValidationError
+from rich.live import Live
+from rich.text import Text
 
 from ._base import _BaseClient
 from ._utils import _drop_none
@@ -30,7 +33,7 @@ class _ParseMixin(_BaseClient):
         file: str,
         parsing_options: Optional[ParsingOptions] = None,
         structured_extraction_options: Optional[
-            List[StructuredExtractionOptions]
+            Union[StructuredExtractionOptions, List[StructuredExtractionOptions]]
         ] = None,
         enrichment_options: Optional[EnrichmentOptions] = None,
         page_classifications: Optional[List[PageClassConfig]] = None,
@@ -120,42 +123,205 @@ class _ParseMixin(_BaseClient):
         resp = await self._arequest("POST", "/parse", json=body)
         return resp.json()["parse_id"]
 
-    def wait_for_completion(self, parse_id: str) -> ParseResult:
+    def wait_for_completion(self, parse_id: str):
         """
         Wait for the completion of a parse operation.
 
-        This method polls the status of a parse operation until it is complete. It checks the status every 5 seconds
-        and returns the final ParseResult once the operation is no longer pending or processing.
+        This methods establishes a connection to the server-sent events (SSE) endpoint for the specified parse ID
+        and listens for updates until the parse operation is complete.
 
         Args:
             parse_id: The ID of the parse operation to wait for. This is the string returned by the parse method.
         """
-        parse = self.get_parsed_result(parse_id)
-        while parse.status in {ParseStatus.PENDING, ParseStatus.PROCESSING}:
-            print("waiting 5 s…")
-            time.sleep(5)
-            parse = self.get_parsed_result(parse_id)
-            print(f"parse status: {parse.status.name.lower()}")
-        return parse
+        status_text = Text("Waiting for completion of parse job.", style="bold")
+        retry_count = 0
+
+        with Live(
+            status_text,
+            refresh_per_second=4,
+            transient=True,
+            redirect_stdout=True,
+            redirect_stderr=True,
+        ) as live:
+            # Print static info above the live line
+            live.console.print(f"Parse ID: {parse_id}")
+
+            def set_status(message: str, style: Optional[str] = None) -> None:
+                live.update(Text(message, style=style), refresh=True)
+
+            def print_line(message: str) -> None:
+                live.console.print(message)
+
+            while retry_count < 5:
+                try:
+                    with connect_sse(
+                        client=self._client,
+                        method="GET",
+                        url=f"parse/{parse_id}",
+                        headers=self._headers(),
+                    ) as sse:
+                        for sse_event in sse.iter_sse():
+                            parse_result = self._handle_sse_event(
+                                sse_event, set_status, print_line
+                            )
+                            if parse_result:
+                                # Final status has already been rendered via set_status
+                                return parse_result
+
+                        live.console.print(
+                            "[yellow]SSE connection ended without completion event[/yellow]"
+                        )
+
+                except (ConnectionError, TimeoutError) as e:
+                    retry_count += 1
+                    live.console.print(
+                        f"[yellow]Connection issue (attempt {retry_count} / 5): {e}[/yellow]"
+                    )
+                    if retry_count < 5:
+                        wait_time = min(2**retry_count, 30)
+                        live.console.print(
+                            f"[yellow]Retrying in {wait_time} seconds...[/yellow]"
+                        )
+                        time.sleep(wait_time)
+
+            live.console.print(
+                "[yellow]Max retries reached. Checking final status...[/yellow]"
+            )
+            # Fetch final status and render a final line
+            final_result = self.get_parsed_result(parse_id)
+            if final_result.status:
+                # Best-effort styling
+                style = (
+                    "green"
+                    if str(final_result.status).lower()
+                    in {"done", "success", "completed"}
+                    else (
+                        "red"
+                        if str(final_result.status).lower() in {"failed", "error"}
+                        else "magenta"
+                    )
+                )
+                set_status(f"Status: {final_result.status}", style)
+            return final_result
 
     async def wait_for_completion_async(self, parse_id: str) -> ParseResult:
         """
         Wait for the completion of a parse operation asynchronously.
 
-        This method polls the status of a parse operation until it is complete. It checks the status every 5 seconds
-        and returns the final ParseResult once the operation is no longer pending or processing.
+        This methods establishes a connection to the server-sent events (SSE) endpoint for the specified parse ID
+        and listens for updates until the parse operation is complete.
 
         Args:
             parse_id: The ID of the parse operation to wait for. This is the string returned by the parse method.
         """
+        status_text = Text("Waiting for completion of parse job.", style="bold")
+        retry_count = 0
 
-        parse = await self.get_parsed_result_async(parse_id)
-        while parse.status in {ParseStatus.PENDING, ParseStatus.PROCESSING}:
-            print("waiting 5 s…")
-            await asyncio.sleep(5)
-            parse = await self.get_parsed_result_async(parse_id)
-            print(f"parse status: {parse.status}")
-        return parse
+        with Live(
+            status_text,
+            refresh_per_second=4,
+            transient=True,
+            redirect_stdout=True,
+            redirect_stderr=True,
+        ) as live:
+            live.console.print(f"Parse ID: {parse_id}")
+
+            def set_status(message: str, style: Optional[str] = None) -> None:
+                live.update(Text(message, style=style), refresh=True)
+
+            def print_line(message: str) -> None:
+                live.console.print(message)
+
+            while retry_count < 5:
+                try:
+                    async with aconnect_sse(
+                        client=self._client,
+                        method="GET",
+                        url=f"parse/{parse_id}",
+                        headers=self._headers(),
+                    ) as sse:
+                        async for sse_event in sse.aiter_sse():
+                            parse_result = self._handle_sse_event(
+                                sse_event, set_status, print_line
+                            )
+                            if parse_result:
+                                return parse_result
+
+                            # Always yield after processing each event
+                            await asyncio.sleep(0)
+
+                        live.console.print(
+                            "[yellow]SSE connection ended without completion event[/yellow]"
+                        )
+                except (ConnectionError, TimeoutError) as e:
+                    retry_count += 1
+                    live.console.print(
+                        f"[yellow]Connection issue (attempt {retry_count} / 5): {e}[/yellow]"
+                    )
+                    if retry_count < 5:
+                        wait_time = min(2**retry_count, 30)
+                        live.console.print(
+                            f"[yellow]Retrying in {wait_time} seconds...[/yellow]"
+                        )
+                        await asyncio.sleep(wait_time)
+
+            live.console.print(
+                "[yellow]Max retries reached. Checking final status...[/yellow]"
+            )
+            final_result = await self.get_parsed_result_async(parse_id)
+            if final_result.status:
+                style = (
+                    "green"
+                    if str(final_result.status).lower()
+                    in {"done", "success", "completed"}
+                    else (
+                        "red"
+                        if str(final_result.status).lower() in {"failed", "error"}
+                        else "magenta"
+                    )
+                )
+                set_status(f"Status: {final_result.status}", style)
+            return final_result
+
+    def _handle_sse_event(
+        self,
+        sse_event: ServerSentEvent,
+        set_status: Callable[[str, Optional[str]], None],
+        print_line: Callable[[str], None],
+    ) -> Optional[ParseResult]:
+        """
+        Handle SSE event and return True if parse is complete (success or failure).
+        """
+        match sse_event.event:
+            case "parse_update":
+                try:
+                    parse_result = ParseResult.model_validate_json(sse_event.data)
+                    set_status(f"Status: {parse_result.status.value}", "magenta")
+                except ValidationError:
+                    set_status(f"Parse update received: {sse_event.data}", "blue")
+
+                return None
+            case "parse_done":
+                parse_result = ParseResult.model_validate_json(sse_event.data)
+                set_status(f"Parse ID: {parse_result.parse_id} done", "green")
+                return parse_result
+            case "parse_failed":
+                parse_result = ParseResult.model_validate_json(sse_event.data)
+                message = (
+                    f"Parse failed ({parse_result.parse_id}): {parse_result.error}"
+                    if parse_result.error
+                    else f"Parse failed ({parse_result.parse_id})"
+                )
+                set_status(message, "red")
+                # Also print a persistent line above the live display so it remains after exit
+                print_line(f"[red]{message}[/red]")
+                return parse_result
+            case "parse_queued":
+                set_status("Parse job waiting in queue.", "yellow")
+                return None
+            case _:
+                set_status(f"Unknown SSE event: {sse_event.event}", "cyan")
+                return None
 
     def parse_and_wait(
         self,
@@ -315,7 +481,7 @@ class _ParseMixin(_BaseClient):
 
         self._request("DELETE", f"parse/{parse.parse_id}")
 
-    def delete_parse_async(self, parse_id: str) -> None:
+    async def delete_parse_async(self, parse_id: str) -> None:
         """
         Delete a parse operation asynchronously.
 
@@ -334,7 +500,7 @@ class _ParseMixin(_BaseClient):
                 "Please wait for the operation to complete before deleting."
             )
 
-        return self._arequest("DELETE", f"parse/{parse.parse_id}")
+        await self._arequest("DELETE", f"parse/{parse.parse_id}")
 
     def list_parse_results(
         self,
@@ -478,7 +644,7 @@ def _create_parse_req(
 
     if file.startswith(("http://", "https://")):
         payload["file_url"] = file
-    elif file.startswith("tensorlake-"):
+    elif file.startswith("tensorlake-") or file.startswith("file_"):
         payload["file_id"] = file
     else:
         payload["raw_text"] = file
