@@ -10,6 +10,7 @@ from ._base import _BaseClient
 from ._datasets import _DatasetMixin
 from ._files import _FilesMixin
 from ._parse import _ParseMixin
+from .models import Region
 
 
 class DocumentAI(_ParseMixin, _FilesMixin, _DatasetMixin, _BaseClient):
@@ -17,5 +18,10 @@ class DocumentAI(_ParseMixin, _FilesMixin, _DatasetMixin, _BaseClient):
     Document AI client for Tensorlake.
     """
 
-    def __init__(self, api_key: Optional[str] = None, server_url: Optional[str] = None):
+    def __init__(
+        self,
+        api_key: Optional[str] = None,
+        server_url: Optional[str] = None,
+        region: Optional[Region] = Region.US,
+    ):
         super().__init__(api_key=api_key, server_url=server_url)
