@@ -1,5 +1,4 @@
 import importlib
-import io
 import json
 import sys
 import tempfile
@@ -7,8 +6,7 @@ import threading
 import time
 import traceback
 import zipfile
-from contextlib import redirect_stderr, redirect_stdout
-from typing import Any, Dict, Generator, Iterator, Optional
+from typing import Dict, Generator, Iterator, List, Optional
 
 import grpc
 
@@ -21,25 +19,27 @@ from tensorlake.functions_sdk.graph_serialization import (
     GraphManifest,
 )
 
+from .blob_store.blob_store import BLOBStore
 from .handlers.check_health.handler import Handler as CheckHealthHandler
 from .handlers.run_function.handler import Handler as RunTaskHandler
 from .info import info_response_kv_args
 from .initialize_request_validator import InitializeRequestValidator
 from .invocation_state.invocation_state_proxy_server import InvocationStateProxyServer
 from .invocation_state.proxied_invocation_state import ProxiedInvocationState
+from .logger import FunctionExecutorLogger
 from .proto.function_executor_pb2 import (
     AwaitTaskProgress,
     AwaitTaskRequest,
     CreateTaskRequest,
     DeleteTaskRequest,
     Empty,
-    FunctionInputs,
     HealthCheckRequest,
     HealthCheckResponse,
     InfoRequest,
     InfoResponse,
     InitializationFailureReason,
     InitializationOutcomeCode,
+    InitializeDiagnostics,
     InitializeRequest,
     InitializeResponse,
     InvocationStateRequest,
@@ -47,95 +47,49 @@ from .proto.function_executor_pb2 import (
     ListTasksRequest,
     ListTasksResponse,
     ProgressUpdate,
-    RunTaskRequest,
-    RunTaskResponse,
     Task,
-    TaskFailureReason,
-    TaskOutcomeCode,
     TaskResult,
 )
 from .proto.function_executor_pb2_grpc import FunctionExecutorServicer
 from .proto.message_validator import MessageValidator
-from .std_outputs_capture import flush_logs, read_till_the_end
-
-
-def _run_task_request_to_function_inputs(run_request: RunTaskRequest) -> FunctionInputs:
-    """Convert RunTaskRequest to FunctionInputs."""
-    request = FunctionInputs()
-
-    if run_request.HasField("function_input"):
-        request.function_input.CopyFrom(run_request.function_input)
-
-    if run_request.HasField("function_init_value"):
-        request.function_init_value.CopyFrom(run_request.function_init_value)
-
-    return request
-
-
-def _run_task_request_to_task(run_request: RunTaskRequest) -> Task:
-    """Convert RunTaskRequest to Task."""
-    return Task(
-        task_id=run_request.task_id,
-        namespace=run_request.namespace,
-        graph_name=run_request.graph_name,
-        graph_version=run_request.graph_version,
-        function_name=run_request.function_name,
-        graph_invocation_id=run_request.graph_invocation_id,
-        allocation_id=run_request.allocation_id,
-        request=_run_task_request_to_function_inputs(run_request),
-    )
-
-
-def _task_result_to_run_task_response(
-    task_id: str, result: TaskResult
-) -> RunTaskResponse:
-    """Convert TaskResult to RunTaskResponse."""
-    response = RunTaskResponse(
-        task_id=task_id,
-        function_outputs=list(result.function_outputs),
-        next_functions=list(result.next_functions),
-        stdout=result.stdout,
-        stderr=result.stderr,
-        is_reducer=result.is_reducer,
-        outcome_code=result.outcome_code,
-        failure_reason=result.failure_reason,
-    )
-
-    if result.HasField("metrics"):
-        response.metrics.CopyFrom(result.metrics)
-
-    if result.HasField("invocation_error_output"):
-        response.invocation_error_output.CopyFrom(result.invocation_error_output)
-
-    return response
+from .user_events import (
+    InitializationEventDetails,
+    log_user_event_initialization_finished,
+    log_user_event_initialization_started,
+)
 
 
 class _TaskExecution:
     def __init__(self):
-        self.complete = False
-        self.thread: threading.Thread | None = None
-        self.updated = threading.Condition()
-        self.progress = ProgressUpdate(current=0.0, total=1.0)
+        self.complete: bool = False
+        self.thread: Optional[threading.Thread] = None
+        self.updated: threading.Condition = threading.Condition()
+        self.progress: ProgressUpdate = ProgressUpdate(current=0.0, total=1.0)
 
 
 class _TaskInfo:
-    def __init__(self, task: Task):
-        self.task = task
-        self.execution = _TaskExecution()
-        self.task.result.Clear()
+    def __init__(self, task: Task, logger: FunctionExecutorLogger):
+        self.task: Task = task
+        self.execution: _TaskExecution = _TaskExecution()
+        self.logger: FunctionExecutorLogger = logger.bind(
+            invocation_id=task.graph_invocation_id,
+            task_id=task.task_id,
+            allocation_id=task.allocation_id,
+        )
 
 
 class Service(FunctionExecutorServicer):
-    def __init__(self, logger: Any):
+    def __init__(self, logger: FunctionExecutorLogger):
         # All the fields are set during the initialization call.
-        self._logger = logger.bind(module=__name__, **info_response_kv_args())
+        self._logger: FunctionExecutorLogger = logger.bind(
+            module=__name__, **info_response_kv_args()
+        )
         self._namespace: Optional[str] = None
         self._graph_name: Optional[str] = None
         self._graph_version: Optional[str] = None
         self._function_name: Optional[str] = None
         self._function_wrapper: Optional[TensorlakeFunctionWrapper] = None
-        self._function_stdout: Optional[io.StringIO] = None
-        self._function_stderr: Optional[io.StringIO] = None
+        self._blob_store: Optional[BLOBStore] = None
         self._graph_metadata: Optional[ComputeGraphMetadata] = None
         self._invocation_state_proxy_server: Optional[InvocationStateProxyServer] = None
         self._check_health_handler: Optional[CheckHealthHandler] = None
@@ -154,6 +108,14 @@ class Service(FunctionExecutorServicer):
         )
         request_validator.check()
 
+        event_details: InitializationEventDetails = InitializationEventDetails(
+            namespace=request.namespace,
+            graph_name=request.graph_name,
+            graph_version=request.graph_version,
+            function_name=request.function_name,
+        )
+        log_user_event_initialization_started(event_details)
+
         self._namespace = request.namespace
         self._graph_name = request.graph_name
         self._graph_version = request.graph_version
@@ -165,14 +127,6 @@ class Service(FunctionExecutorServicer):
             fn=request.function_name,
         )
 
-        # The files don't have paths in filesystem so they get deleted on process exit including crashes.
-        # Using buffered files instead of memory buffers for stdout, stderr puts a natural rate limit on the rate
-        # of writes and allows to not consume expensive memory for function logs.
-        # The same files should be used for stdout and stderr throughout the function lifetime because its code or
-        # dependencies save the files during function creation and the can write to them later while we're running it.
-        self._function_stdout = tempfile.TemporaryFile(mode="w+", encoding="utf-8")
-        self._function_stderr = tempfile.TemporaryFile(mode="w+", encoding="utf-8")
-
         graph_modules_zip_fd, graph_modules_zip_path = tempfile.mkstemp(suffix=".zip")
         with open(graph_modules_zip_fd, "wb") as graph_modules_zip_file:
             graph_modules_zip_file.write(request.graph.data)
@@ -203,54 +157,55 @@ class Service(FunctionExecutorServicer):
                 request.function_name
             ]
 
-            # Flush any logs buffered in memory before doing stdout, stderr capture.
-            # Otherwise our logs logged before this point will end up in the function's stdout capture.
-            flush_logs(self._function_stdout, self._function_stderr)
-            with redirect_stdout(self._function_stdout), redirect_stderr(
-                self._function_stderr
-            ):
-                # Capture output before loading function code.
-                function_module = importlib.import_module(
-                    function_manifest.module_import_name
-                )
-                function_class = getattr(
-                    function_module, function_manifest.class_import_name
-                )
-                # The function is only loaded once per Function Executor. It's important to use a single
-                # loaded function so all the tasks when executed are sharing the same memory. This allows
-                # implementing smart caching in customer code. E.g. load a model into GPU only once and
-                # share the model's file descriptor between all tasks or download function configuration
-                # only once.
-                self._function_wrapper = TensorlakeFunctionWrapper(function_class)
-                # Ensure that whatever outputted by the function gets captured.
-                flush_logs(self._function_stdout, self._function_stderr)
-        except Exception as e:
+            # Capture output before loading function code.
+            function_module = importlib.import_module(
+                function_manifest.module_import_name
+            )
+            function_class = getattr(
+                function_module, function_manifest.class_import_name
+            )
+            # The function is only loaded once per Function Executor. It's important to use a single
+            # loaded function so all the tasks when executed are sharing the same memory. This allows
+            # implementing smart caching in customer code. E.g. load a model into GPU only once and
+            # share the model's file descriptor between all tasks or download function configuration
+            # only once.
+            self._function_wrapper = TensorlakeFunctionWrapper(function_class)
+        except BaseException as e:
             self._logger.error(
                 "function executor service initialization failed",
                 reason="failed to load customer function",
                 duration_sec=f"{time.monotonic() - start_time:.3f}",
                 # Don't log the exception to FE log as it contains customer data
             )
-            formatted_exception: str = "".join(traceback.format_exception(e))
+            log_user_event_initialization_finished(event_details, success=False)
+            # Print the exception to stderr so customer can see it there.
+            traceback.print_exception(e)
             return InitializeResponse(
-                outcome_code=InitializationOutcomeCode.INITIALIZE_OUTCOME_CODE_FAILURE,
+                outcome_code=InitializationOutcomeCode.INITIALIZATION_OUTCOME_CODE_FAILURE,
                 failure_reason=InitializationFailureReason.INITIALIZATION_FAILURE_REASON_FUNCTION_ERROR,
-                stdout=read_till_the_end(self._function_stdout, 0),
-                stderr="\n".join(
-                    [read_till_the_end(self._function_stderr, 0), formatted_exception]
+                diagnostics=InitializeDiagnostics(
+                    function_executor_log=self._logger.read_till_the_end(start=0),
                 ),
             )
 
+        available_cpu_count: int = int(
+            self._graph_metadata.functions[request.function_name].resources.cpus
+        )
+        self._blob_store = BLOBStore(
+            available_cpu_count=available_cpu_count, logger=self._logger
+        )
         # Only pass health checks if FE was initialized successfully.
         self._check_health_handler = CheckHealthHandler(self._logger)
         self._logger.info(
             "initialized function executor service",
             duration_sec=f"{time.monotonic() - start_time:.3f}",
         )
+        log_user_event_initialization_finished(event_details, success=True)
         return InitializeResponse(
-            outcome_code=InitializationOutcomeCode.INITIALIZE_OUTCOME_CODE_SUCCESS,
-            stdout=read_till_the_end(self._function_stdout, 0),
-            stderr=read_till_the_end(self._function_stderr, 0),
+            outcome_code=InitializationOutcomeCode.INITIALIZATION_OUTCOME_CODE_SUCCESS,
+            diagnostics=InitializeDiagnostics(
+                function_executor_log=self._logger.read_till_the_end(start=0),
+            ),
         )
 
     def initialize_invocation_state_server(
@@ -281,28 +236,37 @@ class Service(FunctionExecutorServicer):
         yield from self._invocation_state_proxy_server.run()
         self._invocation_state_proxy_server = None
 
-    def _validate_task(self, task: Task):
-        # Customer function code never raises an exception because we catch all of them and add
-        # their details to the response. We can only get an exception here if our own code failed.
-        # If our code raises an exception the grpc framework converts it into GRPC_STATUS_UNKNOWN
-        # error with the exception message. Differentiating errors is not needed for now.
+    def _validate_new_task(self, task: Task):
+        """Validates the task before creating it.
 
+        Raises ValueError if the task is invalid or is not for this FE.
+        This is internal error due to wrong use of FE protocol.
+        """
         # Validate required fields
-        validator = MessageValidator(task)
-        validator.required_field("namespace")
-        validator.required_field("graph_name")
-        validator.required_field("graph_version")
-        validator.required_field("function_name")
-        validator.required_field("graph_invocation_id")
-        validator.required_field("task_id")
-        validator.required_field("allocation_id")
-        validator.required_field("request")
+        (
+            MessageValidator(task)
+            .required_field("namespace")
+            .required_field("graph_name")
+            .required_field("graph_version")
+            .required_field("function_name")
+            .required_field("graph_invocation_id")
+            .required_field("task_id")
+            .required_field("allocation_id")
+            .required_field("request")
+            .not_set_field("result")
+        )
 
         # Validate task request (input data)
-        request_validator = MessageValidator(task.request)
-        request_validator.required_serialized_object("function_input")
+        (
+            MessageValidator(task.request)
+            .required_blob("function_input_blob")
+            .required_serialized_object_inside_blob("function_input")
+            .optional_blob("function_init_value_blob")
+            .optional_serialized_object_inside_blob("function_init_value")
+            .required_blob("function_outputs_blob")
+            .required_blob("invocation_error_blob")
+        )
 
-        # Fail with internal error as this happened due to wrong task routing to this Server.
         # If we run the wrongly routed task then it can steal data from this Server if it belongs
         # to a different customer.
         if task.namespace != self._namespace:
@@ -322,27 +286,6 @@ class Service(FunctionExecutorServicer):
                 f"This Function Executor is not initialized for this function_name {task.function_name}"
             )
 
-    def run_task(
-        self, request: RunTaskRequest, context: grpc.ServicerContext
-    ) -> RunTaskResponse:
-        # Convert RunTaskRequest to Task for create_task
-        task = _run_task_request_to_task(request)
-        self.create_task(CreateTaskRequest(task=task), context)
-
-        try:
-            for response in self.await_task(
-                AwaitTaskRequest(task_id=request.task_id), context
-            ):
-                last_response = response
-        finally:
-            self.delete_task(DeleteTaskRequest(task_id=request.task_id), context)
-
-        assert last_response.WhichOneof("response") == "task_result"
-        # Convert TaskResult back to RunTaskResponse for backward compatibility
-        return _task_result_to_run_task_response(
-            request.task_id, last_response.task_result
-        )
-
     def check_health(
         self, request: HealthCheckRequest, context: grpc.ServicerContext
     ) -> HealthCheckResponse:
@@ -372,32 +315,25 @@ class Service(FunctionExecutorServicer):
             time.sleep(0)
 
         try:
-            # Run the task handler
-            result = RunTaskHandler(
+            result: TaskResult = RunTaskHandler(
                 task=task_info.task,
                 invocation_state=ProxiedInvocationState(
                     task_info.task.task_id, self._invocation_state_proxy_server
                 ),
                 function_wrapper=self._function_wrapper,
-                function_stdout=self._function_stdout,
-                function_stderr=self._function_stderr,
                 graph_metadata=self._graph_metadata,
                 progress_reporter=progress_reporter,
-                logger=self._logger,
+                blob_store=self._blob_store,
+                logger=task_info.logger,
             ).run()
-
+            # We don't store any large objects in the result so it's okay to copy it.
             task_info.task.result.CopyFrom(result)
-
         except BaseException as e:
-            # Handle any errors by creating a failed task result
-            self._logger.error(
+            # Only exceptions in our code can be raised here so we have to log them.
+            task_info.logger.error(
                 "task execution failed in background thread",
-                task_id=task_info.task.task_id,
                 exc_info=e,
             )
-
-            raise
-
         finally:
             with task_info.execution.updated:
                 task_info.execution.complete = True
@@ -406,29 +342,18 @@ class Service(FunctionExecutorServicer):
     def list_tasks(
         self, request: ListTasksRequest, context: grpc.ServicerContext
     ) -> ListTasksResponse:
+        tasks: List[Task] = []
         with self._tasks_lock:
-            tasks = []
             for task_info in self._tasks.values():
                 with task_info.execution.updated:
-                    # Create a copy of the task without the request field for listing
-                    task_copy = Task()
-                    task_copy.CopyFrom(task_info.task)
-                    task_copy.ClearField(
-                        "request"
-                    )  # Don't return input data when listing
-                    task_copy.ClearField(
-                        "response"
-                    )  # Don't return output data when listing
-                    tasks.append(task_copy)
-
-            return ListTasksResponse(tasks=tasks)
+                    tasks.append(_trimmed_task(task_info.task))
+        return ListTasksResponse(tasks=tasks)
 
     def create_task(
         self, request: CreateTaskRequest, context: grpc.ServicerContext
     ) -> Task:
-        task = request.task
-
-        self._validate_task(task)
+        task: Task = request.task
+        self._validate_new_task(task)
 
         with self._tasks_lock:
             if task.task_id in self._tasks:
@@ -437,7 +362,7 @@ class Service(FunctionExecutorServicer):
                     f"Task {task.task_id} already exists",
                 )
 
-            task_info = _TaskInfo(task)
+            task_info: _TaskInfo = _TaskInfo(task, self._logger)
             self._tasks[task.task_id] = task_info
 
             task_info.execution.thread = threading.Thread(
@@ -447,33 +372,35 @@ class Service(FunctionExecutorServicer):
             )
             task_info.execution.thread.start()
 
-        # Return a minimal task with no optional fields set; we'll
-        # extend this iff the server provides new info.
-        return Task()
+        return _trimmed_task(task)
 
     def await_task(
         self, request: AwaitTaskRequest, context: grpc.ServicerContext
     ) -> Generator[AwaitTaskProgress, None, None]:
         """Wait for task completion and stream progress updates."""
-        task_id = request.task_id
+        task_info: Optional[_TaskInfo] = None
 
         with self._tasks_lock:
-            task_info = self._tasks.get(task_id)
-
-        if task_info is None:
-            context.abort(
-                grpc.StatusCode.NOT_FOUND,
-                f"Task {task_id} not found",
-            )
+            task_info = self._tasks.get(request.task_id)
+            if task_info is None:
+                context.abort(
+                    grpc.StatusCode.NOT_FOUND,
+                    f"Task {request.task_id} not found",
+                )
 
         # Stream progress updates until task completes
-        final_result = None
-        sent_first_progress = False
+        final_result: Optional[TaskResult] = None
+        sent_first_progress: bool = False
         while True:
             with task_info.execution.updated:
                 if task_info.execution.complete:
-                    # Use TaskResult directly
-                    final_result = task_info.task.result
+                    # Don't send default TaskResult() with all fields not set as its meaning is undefined.
+                    # This happens when the function thread finishes with an unexpected error.
+                    final_result = (
+                        task_info.task.result
+                        if task_info.task.HasField("result")
+                        else None
+                    )
                     break
 
                 if sent_first_progress:
@@ -483,16 +410,20 @@ class Service(FunctionExecutorServicer):
 
                 progress = task_info.execution.progress
 
+            # Do all blocking calls outside of the lock.
             if progress:
                 yield AwaitTaskProgress(progress=progress)
 
-        yield AwaitTaskProgress(task_result=final_result)
+        # If the final result doesn't get sent before the stream is closed by FE
+        # then client treats this as a grey failure with unknown exact cause.
+        if final_result is not None:
+            yield AwaitTaskProgress(task_result=final_result)
 
     def delete_task(
         self, request: DeleteTaskRequest, context: grpc.ServicerContext
     ) -> Empty:
         """Delete a task and clean up resources."""
-        task_id = request.task_id
+        task_id: str = request.task_id
 
         with self._tasks_lock:
             task_info = self._tasks.get(task_id)
@@ -513,3 +444,14 @@ class Service(FunctionExecutorServicer):
             self._tasks.pop(task_id, None)
 
         return Empty()
+
+
+def _trimmed_task(task: Task) -> Task:
+    """Returns metadata fields of the task without any large fields like request and result."""
+    task_copy = Task()
+    # We don't store any large objects in the request and result
+    # so it's okay to copy it.
+    task_copy.CopyFrom(task)
+    task_copy.ClearField("request")
+    task_copy.ClearField("result")
+    return task_copy
